@@ -3,6 +3,7 @@ package Pod::POM::Web; # see doc at end of file
 #======================================================================
 use strict;
 use warnings;
+use 5.008;
 no warnings 'uninitialized';
 
 use Pod::POM 0.25;                  # parsing Pod
@@ -14,52 +15,48 @@ use URI;                            # parsing incoming requests
 use URI::QueryParam;                # implements URI->query_form_hash
 use MIME::Types;                    # translate file extension into MIME type
 use Alien::GvaScript 1.021000;      # javascript files
-use Encode::Guess;                  # guessing if pod source is utf8 or latin1
 use Config;                         # where are the script directories
 use Getopt::Long    qw/GetOptions/; # parsing options from command-line
 use Module::Metadata 1.000033;      # get version number from module
+use Encode          qw/decode encode_utf8/;
+use Encode::Detect;
+
 
 #----------------------------------------------------------------------
 # globals
 #---------------------------------------------------------------------
 
-our $VERSION = '1.23';
+our $VERSION = '1.24';
 
 # some subdirs never contain Pod documentation
-my @ignore_toc_dirs = qw/auto unicore/; 
+my @ignore_toc_dirs = qw/auto unicore/;
 
-# filter @INC (don't want '.', nor server_root added by mod_perl)
+# directories for modules -- filter @INC (we don't want '.', nor server_root added by mod_perl)
 my $server_root = eval {Apache2::ServerUtil::server_root()} || "";
 our                # because accessed from Pod::POM::Web::Indexer
-   @search_dirs = grep {!/^\./ && $_ ne $server_root} @INC;
+  @default_module_dirs = grep {!/^\./ && $_ ne $server_root} @INC;
 
 # directories for executable perl scripts
-my @config_script_dirs = qw/sitescriptexp vendorscriptexp scriptdirexp/;
-my @script_dirs        = grep {$_} @Config{@config_script_dirs};
+my @config_script_dirs  = qw/sitescriptexp vendorscriptexp scriptdirexp/;
+my @default_script_dirs = grep {$_} @Config{@config_script_dirs};
 
 # syntax coloring (optional)
-my $coloring_package 
+my $coloring_package
   = eval {require PPI::HTML}              ? "PPI"
-  : eval {require ActiveState::Scineplex} ? "SCINEPLEX" 
+  : eval {require ActiveState::Scineplex} ? "SCINEPLEX"
   : "";
 
-# fulltext indexing (optional)
+# full-text indexing (optional)
 my $no_indexer = eval {require Pod::POM::Web::Indexer} ? 0 : $@;
 
 # CPAN latest version info (tentative, but disabled because CPAN is too slow)
 my $has_cpan = 0; # eval {require CPAN};
 
 # A sequence of optional filters to apply to the source code before
-# running it through Pod::POM. Source code is passed in $_[0] and 
+# running it through Pod::POM. Source code is passed in $_[0] and
 # should be modified in place.
 my @podfilters = (
 
-  # AnnoCPAN must be first in the filter list because 
-  # it uses the MD5 of the original source
-  eval {require AnnoCPAN::Perldoc::Filter} 
-    ? sub {$_[0] = AnnoCPAN::Perldoc::Filter->new->filter($_[0])} 
-    : (),
-
   # Pod::POM fails to parse correctly when there is an initial blank line
   sub { $_[0] =~ s/\A\s*// },
 
@@ -82,12 +79,12 @@ sub import {
   my ($package, $filename) = caller;
 
   no strict 'refs';
-  *{'main::server'} = sub {$class->server(@_)} 
+  *{'main::server'} = sub {$class->server(@_)}
     if $package eq 'main' and $filename eq '-e';
 }
 
 #----------------------------------------------------------------------
-# main entry point
+# CLASS METHODS -- main entry point
 #----------------------------------------------------------------------
 
 sub server { # builtin HTTP server; unused if running under Apache
@@ -99,7 +96,7 @@ sub server { # builtin HTTP server; unused if running under Apache
   my $daemon = HTTP::Daemon->new(LocalPort => $port,
                                  ReuseAddr => 1) # patch by CDOLAN
     or die "could not start daemon on port $port";
-  print STDERR "Please contact me at: <URL:", $daemon->url, ">\n";
+  print STDERR "$class listening for requests at URL: ", $daemon->url, "\n";
 
   # main server loop
   while (my $client_connection = $daemon->accept) {
@@ -117,23 +114,49 @@ sub server { # builtin HTTP server; unused if running under Apache
 
 
 sub _options_from_cmd_line {
-  GetOptions(\my %options, qw/port=i page_title|title=s/);
+  GetOptions \my %options,
+    'port=i',
+    'page_title|title=s',
+    'module_dirs|mdirs=s@{,}',
+    'script_dirs|sdirs=s@{,}',
+    ;
+
+  push @{$options{module_dirs}}, @default_module_dirs;
+  push @{$options{script_dirs}}, @default_script_dirs;
+
   $options{port} ||= $ARGV[0] if @ARGV; # backward support for old API
   return \%options;
 }
 
-
 sub handler : method  {
-  my ($class, $request, $response, $options) = @_; 
-  my $self = $class->new($request, $response, $options);
-  eval { $self->dispatch_request(); 1}
-    or $self->send_content({content => $@, code => 500});  
+  my ($class, $request, $response, $options) = @_;
+
+  my $handler_obj = $class->new($request, $response, $options);
+
+  eval { $handler_obj->dispatch_request(); 1}
+    or do {
+      my $error = $@;
+      if ($error =~ /No file for '(.*)'/) {
+        $handler_obj->send_content({content => $handler_obj->_no_such_module($1),
+                                    code    => 403});
+      }
+      else {
+        $handler_obj->send_content({content => $error,
+                                    code    => 500});
+      }
+  };
+
   return 0; # Apache2::Const::OK;
 }
 
 
+#----------------------------------------------------------------------
+# CONSTRUCTOR for the "handler object" -- instantiated at each request
+#----------------------------------------------------------------------
+
+
 sub new  {
-  my ($class, $request, $response, $options) = @_; 
+  my ($class, $request, $response, $options) = @_;
   $options ||= {};
   my $self = {%$options};
 
@@ -177,30 +200,38 @@ sub new  {
   bless $self, $class;
 }
 
+#----------------------------------------------------------------------
+# INSTANCE METHODS for the "handler object"
+#----------------------------------------------------------------------
 
 
+sub module_dirs {@{shift->{module_dirs}}}
+sub script_dirs {@{shift->{script_dirs}}}
 
-sub dispatch_request { 
+sub dispatch_request {
   my ($self) = @_;
   my $path_info = $self->{path};
 
   # security check : no outside directories
   $path_info =~ m[(\.\.|//|\\|:)] and die "illegal path: $path_info";
 
-  $path_info =~ s[^/][] or return $self->index_frameset; 
+  $path_info =~ s[^/][] or return $self->index_frameset;
   for ($path_info) {
-    /^$/               and return $self->index_frameset; 
-    /^index$/          and return $self->index_frameset; 
-    /^toc$/            and return $self->main_toc; 
+    /^$/               and return $self->index_frameset;
+    /^index$/          and return $self->index_frameset;
+    /^toc$/            and return $self->main_toc;
     /^toc\/(.*)$/      and return $self->toc_for($1);   # Ajax calls
     /^script\/(.*)$/   and return $self->serve_script($1);
     /^search$/         and return $self->dispatch_search;
     /^source\/(.*)$/   and return $self->serve_source($1);
 
     # for debugging
-    /^_dirs$/          and return $self->send_html(join "<br>", @search_dirs);
+    /^_dirs$/          and return $self->send_html(
+      join "<br>", "<b>Modules</b>" => $self->module_dirs,
+                   "<b>Scripts</b>" => $self->script_dirs,
+      );
 
-    # file extension : passthrough 
+    # file extension : passthrough
     /\.(\w+)$/         and return $self->serve_file($path_info, $1);
 
     #otherwise
@@ -253,7 +284,9 @@ sub serve_source {
   my $display_text;
 
   foreach my $file (@files) {
-    my $text = $self->slurp_file($file, ":crlf");
+    my $text = decode("Detect", $self->slurp_file($file, ":raw"));
+    $text =~ s/\r\n/\n/g;
+
     my $view = $self->mk_view(
       line_numbering  => $params->{lines},
       syntax_coloring => ($params->{coloring} ? $coloring_package : "")
@@ -281,13 +314,13 @@ __EOHTML__
 <a href="$self->{root_url}/$path" style="float:right">Doc</a>
 __EOHTML__
 
-  return $self->send_html(<<__EOHTML__, $mtime);
+  my $html = <<__EOHTML__;
 <html>
 <head>
   <title>Source of $path</title>
   <link href="$self->{root_url}/Alien/GvaScript/lib/GvaScript.css" rel="stylesheet" type="text/css">
   <link href="$self->{root_url}/Pod/POM/Web/lib/PodPomWeb.css" rel="stylesheet" type="text/css">
-  <style> 
+  <style>
     PRE {border: none; background: none}
     FORM {float: right; font-size: 70%; border: 1px solid}
   </style>
@@ -302,13 +335,16 @@ $display_text
 </body>
 </html>
 __EOHTML__
+
+  $self->send_html($html, $mtime);
+
 }
 
 
 sub serve_file {
   my ($self, $path, $extension) = @_;
 
-  my $fullpath = firstval {-f $_} map {"$_/$path"} @search_dirs
+  my $fullpath = firstval {-f $_} map {"$_/$path"} $self->module_dirs
     or die "could not find $path";
 
   my $mime_type = MIME::Types->new->mimeTypeOf($extension);
@@ -331,11 +367,11 @@ sub serve_pod {
   my @sources = $self->find_source($path) or die "No file for '$path'";
   my $mtime   = max map {(stat $_)[9]} @sources;
   my $content = $path =~ /\bperltoc\b/
-                   ? $self->fake_perltoc 
+                   ? $self->fake_perltoc
                    : $self->slurp_file($sources[0], ":crlf");
 
   (my $mod_name = $path) =~ s[/][::]g;
-  my $version = @sources > 1 
+  my $version = @sources > 1
     ? $self->parse_version($self->slurp_file($sources[-1], ":crlf"), $mod_name)
     : $self->parse_version($content, $mod_name);
 
@@ -344,7 +380,7 @@ sub serve_pod {
   }
 
   # special handling for perlfunc: change initial C<..> to hyperlinks
-  if ($path =~ /\bperlfunc$/) { 
+  if ($path =~ /\bperlfunc$/) {
     my $sub = sub {my $txt = shift; $txt =~ s[C<(.*?)>][C<L</$1>>]g; $txt};
     $content =~ s[(Perl Functions by Category)(.*?)(Alphabetical Listing)]
                  [$1 . $sub->($2) . $3]es;
@@ -361,12 +397,12 @@ sub serve_pod {
   my $html = $view->print($pom);
 
   # again special handling for perlfunc : ids should be just function names
-  if ($path =~ /\bperlfunc$/) { 
+  if ($path =~ /\bperlfunc$/) {
     $html =~ s/li id="(.*?)_.*?"/li id="$1"/g;
   }
 
   # special handling for 'perl' : hyperlinks to man pages
-  if ($path =~ /\bperl$/) { 
+  if ($path =~ /\bperl$/) {
     my $sub = sub {my $txt = shift;
                    $txt =~ s[(perl\w+)]
                             [<a href="$self->{root_url}/$1">$1</a>]g;
@@ -394,7 +430,7 @@ sub serve_script {
   my $fullpath;
 
  DIR:
-  foreach my $dir (@script_dirs) {
+  foreach my $dir ($self->script_dirs) {
     foreach my $ext ("", ".pl", ".bat") {
       $fullpath = "$dir/$path$ext";
       last DIR if -f $fullpath;
@@ -427,7 +463,7 @@ sub find_source {
   # serving a script ?    # TODO : factorize common code with serve_script
   if ($path =~ s[^scripts/][]) {
   DIR:
-    foreach my $dir (@script_dirs) {
+    foreach my $dir ($self->script_dirs) {
       foreach my $ext ("", ".pl", ".bat") {
         -f "$dir/$path$ext" or next;
         return ("$dir/$path$ext");
@@ -437,9 +473,9 @@ sub find_source {
   }
 
   # otherwise, serving a module
-  foreach my $prefix (@search_dirs) {
-    my @found = grep  {-f} ("$prefix/$path.pod", 
-                            "$prefix/$path.pm", 
+  foreach my $prefix ($self->module_dirs) {
+    my @found = grep  {-f} ("$prefix/$path.pod",
+                            "$prefix/$path.pm",
                             "$prefix/pod/$path.pod",
                             "$prefix/pods/$path.pod");
     return @found if @found;
@@ -461,6 +497,29 @@ sub pod2pom {
   return $pom;
 }
 
+sub _no_such_module {
+  my ($self, $module) = @_;
+
+  $module =~ s!/!::!g;
+  $module =~ s/([&<>"])/$escape_entity{$1}/g;
+  return  <<__EOHTML__;
+<html>
+  <head>
+    <title>$module not found</title>
+  </head>
+  <body>
+    <h1>$module not found</h1>
+    <p>
+      The module <code>$module</code> could not be found on this server.
+      It may not be installed locally. Please try 
+      <a href='https://metacpan.org/pod/$module'>$module on Metacpan</a>.
+    </p>
+  </body>
+</html>
+__EOHTML__
+}
+
+
 #----------------------------------------------------------------------
 # tables of contents
 #----------------------------------------------------------------------
@@ -470,7 +529,7 @@ sub toc_for { # partial toc (called through Ajax)
   my ($self, $prefix) = @_;
 
   # special handling for builtin paths
-  for ($prefix) { 
+  for ($prefix) {
     /^perldocs$/ and return $self->toc_perldocs;
     /^pragmas$/  and return $self->toc_pragmas;
     /^scripts$/  and return $self->toc_scripts;
@@ -510,7 +569,7 @@ sub toc_pragmas {
   my ($self) = @_;
 
   my $entries  = $self->find_entries_for("");    # files found at root level
-  delete $entries->{$_} for @ignore_toc_dirs, qw/pod pods inc/; 
+  delete $entries->{$_} for @ignore_toc_dirs, qw/pod pods inc/;
   delete $entries->{$_} for grep {/^perl/ or !/^[[:lower:]]/} keys %$entries;
 
   return $self->send_html($self->htmlize_entries($entries));
@@ -523,7 +582,7 @@ sub toc_scripts {
   my %scripts;
 
   # gather all scripts and group them by initial letter
-  foreach my $dir (@script_dirs) {
+  foreach my $dir ($self->script_dirs) {
     opendir my $dh, $dir or next;
   NAME:
     foreach my $name (readdir $dh) {
@@ -561,7 +620,7 @@ sub find_entries_for {
 
   my %entries;
 
-  foreach my $root_dir (@search_dirs) {
+  foreach my $root_dir ($self->module_dirs) {
     my $dirname = $prefix ? "$root_dir/$prefix" : $root_dir;
     opendir my $dh, $dirname or next;
     foreach my $name (readdir $dh) {
@@ -571,7 +630,7 @@ sub find_entries_for {
       my $has_pod = $name =~ s/\.(pm|pod)$//;
 
       # skip if this subdir is a member of @INC (not a real module namespace)
-      next if $is_dir and grep {m[^\Q$dirname/$name\E]} @search_dirs;
+      next if $is_dir and grep {m[^\Q$dirname/$name\E]} $self->module_dirs;
 
       if ($is_dir || $has_pod) { # found a TOC entry
         $entries{$name}{node} = $prefix ? "$prefix/$name" : $name;
@@ -611,20 +670,20 @@ sub htmlize_perldocs {
     while ($content =~ /^\s*(perl\S*?)\s*\t(.+)/gm) {
       my ($ref, $descr) = ($1, $2);
       my $entry = delete $perldocs->{$ref} or next;
-      push @leaves, {label => $ref, 
+      push @leaves, {label => $ref,
                      href  => $entry->{node},
                      attrs => qq{id='$ref' title='$descr'}};
     }
     # sort and transform into HTML
-    @leaves = map {leaf(%$_)} 
+    @leaves = map {leaf(%$_)}
               sort {$a->{label} cmp $b->{label}} @leaves;
-    $html .= closed_node(label   => $title, 
+    $html .= closed_node(label   => $title,
                          content => join("\n", @leaves));
   }
 
   # maybe some remaining pages
   if (keys %$perldocs) {
-    $html .= closed_node(label   => 'Unclassified', 
+    $html .= closed_node(label   => 'Unclassified',
                          content => $self->htmlize_entries($perldocs));
   }
 
@@ -641,7 +700,7 @@ sub htmlize_entries {
     my $entry = $entries->{$name};
     (my $id = $entry->{node}) =~ s[/][::]g;
     my %args = (class => 'TN_leaf',
-                label => $name, 
+                label => $name,
                 attrs => qq{id='$id'});
     if ($entry->{dir}) {
       $args{class}  = 'TN_node TN_closed';
@@ -663,7 +722,7 @@ sub get_abstract {
 
 
 
-sub main_toc { 
+sub main_toc {
   my ($self) = @_;
 
   # initial page to open
@@ -707,9 +766,9 @@ sub main_toc {
 <html>
 <head>
   <base target="contentFrame">
-  <link href="$self->{root_url}/Alien/GvaScript/lib/GvaScript.css" 
+  <link href="$self->{root_url}/Alien/GvaScript/lib/GvaScript.css"
         rel="stylesheet" type="text/css">
-  <link href="$self->{root_url}/Pod/POM/Web/lib/PodPomWeb.css" 
+  <link href="$self->{root_url}/Pod/POM/Web/lib/PodPomWeb.css"
         rel="stylesheet" type="text/css">
   <script src="$self->{root_url}/Alien/GvaScript/lib/prototype.js"></script>
   <script src="$self->{root_url}/Alien/GvaScript/lib/GvaScript.js"></script>
@@ -728,7 +787,7 @@ sub main_toc {
       // compute available height -- comes either from body or documentElement,
       // depending on browser and on compatibility mode !!
       var doc_el_height = document.documentElement.clientHeight;
-      var avail_height 
+      var avail_height
         = (Prototype.Browser.IE && doc_el_height) ? doc_el_height
                                                   : document.body.clientHeight;
 
@@ -746,14 +805,14 @@ sub main_toc {
       first_node = rest.shift();
 
       // build a handler for "onAfterLoadContent" (closure on first_node/rest)
-      var open_or_select_next = function() { 
+      var open_or_select_next = function() {
 
         // delete handler that might have been placed by previous call
         delete treeNavigator.onAfterLoadContent;
 
-        // 
+        //
         if (rest.length > 0) {
-          open_nodes(first_node, rest) 
+          open_nodes(first_node, rest)
         }
         else {
           treeNavigator.openEnclosingNodes(\$(first_node));
@@ -770,7 +829,7 @@ sub main_toc {
         treeNavigator.onAfterLoadContent = open_or_select_next;
         treeNavigator.open(node);
       }
-      // otherwise just a direct call 
+      // otherwise just a direct call
       else {
         open_or_select_next();
       }
@@ -783,14 +842,14 @@ sub main_toc {
       // build array of intermediate nodes (i.e "Foo", "Foo::Bar", etc.)
       var parts = entry.split(new RegExp("/|::"));
       var accu = '';
-      var sequence = parts.map(function(e) { 
+      var sequence = parts.map(function(e) {
          accu = accu ? (accu + "::" + e) : e;
          return accu;
         });
 
       // choose id of first_node by analysis of entry
       var initial = entry.substr(0, 1);
-      var first_node  
+      var first_node
 
         // CASE module (starting with uppercase)
         = (initial <= 'Z')           ? (initial + ":")
@@ -808,28 +867,28 @@ sub main_toc {
 
     function setup() {
 
-      treeNavigator 
+      treeNavigator
         = new GvaScript.TreeNavigator('TN_tree', {tabIndex:-1});
 
       completers.perlfunc = new GvaScript.AutoCompleter(
-             perlfuncs, 
-             {minimumChars: 1, 
-              minWidth: 100, 
-              offsetX: -20, 
+             perlfuncs,
+             {minimumChars: 1,
+              minWidth: 100,
+              offsetX: -20,
               autoSuggestDelay: 400});
       completers.perlfunc.onComplete = submit_on_event;
 
       completers.perlvar = new GvaScript.AutoCompleter(
-             perlvars, 
-             {minimumChars: 1, 
-              minWidth: 100, 
-              offsetX: -20, 
+             perlvars,
+             {minimumChars: 1,
+              minWidth: 100,
+              offsetX: -20,
               autoSuggestDelay: 400});
       completers.perlvar.onComplete = submit_on_event;
 
       if (!no_indexer) {
         completers.modlist  = new GvaScript.AutoCompleter(
-             "search?source=modlist&search=", 
+             "search?source=modlist&search=",
              {minimumChars: 2, minWidth: 100, offsetX: -20, typeAhead: false});
         completers.modlist.onComplete = submit_on_event;
       }
@@ -839,7 +898,7 @@ sub main_toc {
     }
 
     document.observe('dom:loaded', setup);
-    window.onresize = resize_tree_navigator; 
+    window.onresize = resize_tree_navigator;
     // Note: observe('resize') doesn't work. Why ?
 
     function displayContent(event) {
@@ -858,7 +917,7 @@ sub main_toc {
        case 0: completers.perlfunc.autocomplete(input); break;
        case 1: completers.perlvar.autocomplete(input); break;
        case 3: if (!no_indexer)
-                 completers.modlist.autocomplete(input); 
+                 completers.modlist.autocomplete(input);
                break;
      }
    }
@@ -868,7 +927,7 @@ sub main_toc {
   <style>
    .small_title {color: midnightblue; font-weight: bold; padding: 0 3 0 3}
    FORM     {margin:0px}
-   BODY     {margin:0px; font-size: 70%; overflow-x: hidden} 
+   BODY     {margin:0px; font-size: 70%; overflow-x: hidden}
    DIV      {margin:0px; width: 100%}
    #TN_tree {overflow-y:scroll; overflow-x: hidden}
   </style>
@@ -876,7 +935,7 @@ sub main_toc {
 <body>
 
 <div id='toc_frame_top'>
-<div class="small_title" 
+<div class="small_title"
      style="text-align:center;border-bottom: 1px solid">
 Perl Documentation
 </div>
@@ -891,9 +950,9 @@ Perl Documentation
       <option>perlvar</option>
       <option>perlfaq</option>
       <option>modules</option>
-      <option>fulltext</option>
+      <option>full-text</option>
      </select><br>
-<span class="small_title">&nbsp;for&nbsp;</span><input 
+<span class="small_title">&nbsp;for&nbsp;</span><input
          name="search" size="15"
          autocomplete="off"
          onfocus="maybe_complete(this)">
@@ -903,7 +962,7 @@ Perl Documentation
      style="border-bottom: 1px solid">Browse</div>
 </div>
 
-<!-- In principle the tree navigator below would best belong in a 
+<!-- In principle the tree navigator below would best belong in a
      different frame, but instead it's in a div because the autocompleter
      from the form above sometimes needs to overlap the tree nav. -->
 <div id='TN_tree' onPing='displayContent'>
@@ -929,11 +988,11 @@ sub dispatch_search {
                 perlvar       => 'perlvar',
                 perlfaq       => 'perlfaq',
                 modules       => 'serve_pod',
-                fulltext      => 'fulltext',
+                full_text     => 'full_text',
                 modlist       => 'modlist',
                 }->{$source}  or die "cannot search in '$source'";
 
-  if ($method =~ /fulltext|modlist/ and $no_indexer) {
+  if ($method =~ /full_text|modlist/ and $no_indexer) {
     die "<p>this method requires <b>Search::Indexer</b></p>"
       . "<p>please ask your system administrator to install it</p>"
       . "(<small>error message : $no_indexer</small>)";
@@ -954,7 +1013,7 @@ sub perlfunc_items {
       or die "'perlfunc.pod' does not seem to be installed on this system";
     my $funcpom   = $self->pod2pom($funcpod);
     my ($description) = grep {$_->title eq 'DESCRIPTION'} $funcpom->head1;
-    my ($alphalist)   
+    my ($alphalist)
       = grep {$_->title =~ /^Alphabetical Listing/i} $description->head2;
     @_perlfunc_items = $alphalist->over->[0]->item;
   };
@@ -1048,7 +1107,7 @@ sub perlfaq {
 
   my $view = $self->mk_view(path => "perlfaq/$faq_entry");
 
- FAQ: 
+ FAQ:
   for my $num (1..9) {
     my $faq = "perlfaq$num";
     my ($faqpod) = $self->find_source($faq)
@@ -1072,7 +1131,7 @@ sub perlfaq {
   <script src="$self->{root_url}/Alien/GvaScript/lib/GvaScript.js"></script>
   <script>
     var treeNavigator;
-    function setup() {  
+    function setup() {
       treeNavigator = new GvaScript.TreeNavigator('TN_tree');
     }
     window.onload = setup;
@@ -1115,16 +1174,15 @@ sub send_html {
   $html =~ s[<head>]
             [<head>\n<meta http-equiv="X-UA-Compatible" content="IE=edge">];
 
-  $self->send_content({content => $html, code => 200, mtime => $mtime});
+  $self->send_content({content => encode_utf8($html), code => 200, mtime => $mtime, charset => 'UTF-8'});
 }
 
 
 
 sub send_content {
   my ($self, $args) = @_;
-  my $encoding  = guess_encoding($args->{content}, qw/ascii utf8 latin1/);
-  my $charset   = ref $encoding ? $encoding->name : "";
-     $charset   =~ s/^ascii/US-ascii/; # Firefox insists on that imperialist name
+
+  my $charset   = $args->{charset};
   my $length    = length $args->{content};
   my $mime_type = $args->{mime_type} || "text/html";
      $mime_type .= "; charset=$charset" if $charset and $mime_type =~ /html/;
@@ -1174,7 +1232,7 @@ sub generic_node {
   $args{attrs}       &&= " $args{attrs}";
   $args{content}     ||= "";
   $args{content}     &&= qq{<div class="TN_content">$args{content}</div>};
-  my ($default_label_tag, $label_attrs) 
+  my ($default_label_tag, $label_attrs)
     = $args{href} ? ("a",    qq{ href='$args{href}'})
                   : ("span", ""                     );
   $args{label_tag}   ||= $default_label_tag;
@@ -1208,7 +1266,7 @@ sub leaf {
 
 sub slurp_file {
   my ($self, $file, $io_layer) = @_;
-  open my $fh, $file or die "open $file: $!";
+  open my $fh, "<", $file or die "open $file: $!";
   binmode($fh, $io_layer) if $io_layer;
   local $/ = undef;
   return <$fh>;
@@ -1273,7 +1331,7 @@ sub view_seq_text {
 
 
 # SUPER::view_seq_link needs some adaptations
-sub view_seq_link { 
+sub view_seq_link {
     my ($self, $link) = @_;
 
     # we handle the L<link_text|...> syntax here, because we also want
@@ -1292,7 +1350,7 @@ sub view_seq_link {
     $url =~ s[#(.*)]['#' . _title_to_id($1)]e unless $is_external_resource;
 
     # if explicit link_text given by client, take that as label, unchanged
-    if ($link_text) { 
+    if ($link_text) {
       $label = $link_text;
     }
     # if "$page/$section", replace by "$section in $page"
@@ -1318,18 +1376,11 @@ sub view_item {
   my ($self, $item) = @_;
 
   my $title = eval {$item->title->present($self)} || "";
-     $title = "" if $title =~ /^\s*\*\s*$/; 
+     $title = "" if $title =~ /^\s*\*\s*$/;
 
   my $class = "";
-  my $id    = "";
-
-  if ($title =~ /^AnnoCPAN/) {
-    $class = " class='AnnoCPAN'";
-  }
-  else {
-    $id   = _title_to_id($title);
-    $id &&= qq{ id="$id"};
-  }
+  my $id    = _title_to_id($title);
+  $id &&= qq{ id="$id"};
 
   my $content = $item->content->present($self);
   $title   = qq{<b>$title</b>} if $title;
@@ -1373,12 +1424,12 @@ sub view_pod {
     = ("") x 6;
   if (my $mod_name = $self->{mod_name}) {
 
-    # version 
-    $version = $self->{version} ? "v. $self->{version}, " : ""; 
+    # version
+    $version = $self->{version} ? "v. $self->{version}, " : "";
 
     # is this module in Perl core ?
     $core_release = Module::CoreList->first_release($mod_name) || "";
-    $orig_version 
+    $orig_version
       = $Module::CoreList::version{$core_release}{$mod_name} || "";
     $orig_version &&= "v. $orig_version ";
     $core_release &&= "; ${orig_version}entered Perl core in $core_release";
@@ -1386,22 +1437,20 @@ sub view_pod {
     # hyperlinks to various internet resources
     $module_refs = qq{<br>
      <a href="https://metacpan.org/pod/$mod_name"
-        target="_blank">meta::cpan</a> |
-     <a href="http://www.annocpan.org/?mode=search&field=Module&name=$mod_name"
-        target="_blank">Anno</a>
+        target="_blank">meta::cpan</a>
     };
 
     if ($has_cpan) {
       my $mod = CPAN::Shell->expand("Module", $mod_name);
       if ($mod) {
         my $cpan_version = $mod->cpan_version;
-        $cpan_info = "; CPAN has v. $cpan_version" 
+        $cpan_info = "; CPAN has v. $cpan_version"
           if $cpan_version ne $self->{version};
       }
     }
   }
 
-  my $toc = $self->make_toc($pom, 0); 
+  my $toc = $self->make_toc($pom, 0);
 
   return <<__EOHTML__
 <html>
@@ -1413,9 +1462,9 @@ sub view_pod {
   <script src="$self->{root_url}/Alien/GvaScript/lib/GvaScript.js"></script>
   <script>
     var treeNavigator;
-    function setup() {  
+    function setup() {
       new GvaScript.TreeNavigator(
-         'TN_tree', 
+         'TN_tree',
          {selectFirstNode: (location.hash ? false : true),
           tabIndex: 0}
       );
@@ -1441,14 +1490,14 @@ sub view_pod {
 
     #ref_box {
       clear: right;
-      float: right; 
-      text-align: right; 
+      float: right;
+      text-align: right;
       font-size: 80%;
     }
     #title_descr {
        clear: right;
-       float: right; 
-       font-style: italic; 
+       float: right;
+       font-style: italic;
        margin-top: 8px;
        margin-bottom: 8px;
        padding: 5px;
@@ -1549,7 +1598,7 @@ sub PPI_coloring {
   my $ppi = PPI::HTML->new();
   my $html = $ppi->html(\$text);
 
-  if ($html) { 
+  if ($html) {
     $html =~ s/<br>//g;
     return $html;
   }
@@ -1563,8 +1612,8 @@ sub PPI_coloring {
 sub SCINEPLEX_coloring {
   my ($self, $text) = @_;
   eval {
-    $text = ActiveState::Scineplex::Annotate($text, 
-                                             'perl', 
+    $text = ActiveState::Scineplex::Annotate($text,
+                                             'perl',
                                              outputFormat => 'html');
   };
   return $text;
@@ -1618,11 +1667,11 @@ Pod::POM::Web - HTML Perldoc server
 
 L<Pod::POM::Web> is a Web application for browsing
 the documentation of Perl components installed
-on your local machine. Since pages are dynamically 
+on your local machine. Since pages are dynamically
 generated, they are always in sync with code actually
-installed. 
+installed.
 
-The application offers 
+The application offers
 
 =over
 
@@ -1653,7 +1702,7 @@ search through L<perlfaq> headers
 
 =item *
 
-fulltext search, including names of Perl variables
+full-text search, including names of Perl variables
 (this is an optional feature -- see section L</"Optional features">).
 
 =item *
@@ -1683,7 +1732,7 @@ Explorer 8.0, Firefox 3.5, Google Chrome 3.0 and Safari 4.0.4.
 
 =head1 USAGE
 
-Usage is described in a separate document 
+Usage is described in a separate document
 L<Pod::POM::Web::Help>.
 
 =head1 INSTALLATION
@@ -1696,7 +1745,7 @@ the web server :
 
 =head3 As a mod_perl service
 
-If you have Apache2 with mod_perl 2.0, then edit your 
+If you have Apache2 with mod_perl 2.0, then edit your
 F<perl.conf> as follows :
 
   PerlModule Apache2::RequestRec
@@ -1708,7 +1757,7 @@ F<perl.conf> as follows :
 
 Then navigate to URL L<http://localhost/perldoc>.
 
-=head3 As a cgi-bin script 
+=head3 As a cgi-bin script
 
 Alternatively, you can run this application as a cgi-script
 by writing a simple file F<perldoc> in your C<cgi-bin> directory,
@@ -1718,7 +1767,7 @@ containing :
   use Pod::POM::Web;
   Pod::POM::Web->handler;
 
-Make this script executable, 
+Make this script executable,
 then navigate to URL L<http://localhost/cgi-bin/perldoc>.
 
 The same can be done for running under mod_perl Registry
@@ -1764,8 +1813,8 @@ want them to have distinct titles. This can be done like this:
 
 =head2 Note about security
 
-This application is intended as a power tool for Perl developers, 
-not as an Internet application. It will give access to any file 
+This application is intended as a power tool for Perl developers,
+not as an Internet application. It will give access to any file
 installed under your C<@INC> path or Apache C<lib/perl> directory
 (but not outside of those directories);
 so it is probably a B<bad idea>
@@ -1777,17 +1826,17 @@ to put it on a public Internet server.
 =head3 Syntax coloring
 
 Syntax coloring improves readability of code excerpts.
-If your Perl distribution is from ActiveState, then 
-C<Pod::POM::Web> will take advantage 
+If your Perl distribution is from ActiveState, then
+C<Pod::POM::Web> will take advantage
 of the L<ActiveState::Scineplex> module
 which is already installed on your system. Otherwise,
 you need to install L<PPI::HTML>, available from CPAN.
 
-=head3 Fulltext indexing
+=head3 Full-text indexing
 
 C<Pod::POM::Web> can index the documentation and source code
-of all your installed modules, including Perl variable names, 
-C<Names:::Of::Modules>, etc. To use this feature you need to 
+of all your installed modules, including Perl variable names,
+C<Names:::Of::Modules>, etc. To use this feature you need to
 
 =over
 
@@ -1802,52 +1851,25 @@ build the index as described in L<Pod::POM::Web::Indexer> documentation.
 =back
 
 
-
-=head3 AnnoCPAN comments
-
-The website L<http://annocpan.org/> lets people add comments to the
-documentation of CPAN modules.  The AnnoCPAN database is freely
-downloadable and can be easily integrated with locally installed
-modules via runtime filtering.
-
-If you want AnnoCPAN comments to show up in Pod::POM::Web, do the following:
-
-=over
-
-=item *
-
-install L<AnnoCPAN::Perldoc> from CPAN;
-
-=item *
-
-download the database from L<http://annocpan.org/annopod.db> and save
-it as F<$HOME/.annopod.db> (see the documentation in the above module
-for more details).  You may also like to try
-L<AnnoCPAN::Perldoc::SyncDB> which is a crontab-friendly tool for
-periodically downloading the AnnoCPAN database.
-
-=back
-
-
 =head1 HINTS TO POD AUTHORING
 
 =head2 Images
 
-The Pod::Pom::Web server also serves non-pod files within the C<@INC> 
-hierarchy. This is useful for example to include images in your 
-documentation, by inserting chunks of HTML as follows : 
+The Pod::Pom::Web server also serves non-pod files within the C<@INC>
+hierarchy. This is useful for example to include images in your
+documentation, by inserting chunks of HTML as follows :
 
   =for html
     <img src="pretty_diagram.jpg">
 
-or 
+or
 
   =for html
     <object type="image/svg+xml" data="try.svg" width="640" height="480">
     </object>
 
-Here it is assumed that auxiliary files C<pretty_diagram.jpg> or 
-C<try.svg> are in the same directory than the POD source; but 
+Here it is assumed that auxiliary files C<pretty_diagram.jpg> or
+C<try.svg> are in the same directory than the POD source; but
 of course relative or absolute links can be used.
 
 
@@ -1860,11 +1882,35 @@ of course relative or absolute links can be used.
 
 Public entry point for serving a request. Objects C<$request> and
 C<$response> are specific to the hosting HTTP server (modperl, HTTP::Daemon
-or cgi-bin); C<$options> is a hashref that currently contains
-only one possible entry : C<page_title>, for specifying the HTML title
+or cgi-bin); C<$options> is a hashref that can contain 
+
+=over 
+
+=item C<page_title>
+
+for specifying the HTML title
 of the application (useful if you run several concurrent instances
 of Pod::POM::Web).
 
+=item C<port>
+
+the HTTP port listening for requests
+
+=item C<module_dirs>
+
+directories for searching for modules,
+in addition to the standard ones installed with your perl executable.
+
+
+=item <script_dirs>
+
+additional directories for searching for scripts
+
+=back
+
+
+
+
 =head2 server
 
   Pod::POM::Web->server($port, $options);
@@ -1891,7 +1937,7 @@ This web application was deeply inspired by :
 
 =item *
 
-the structure of HTML Perl documentation released with 
+the structure of HTML Perl documentation released with
 ActivePerl (L<http://www.activeperl.com/ASPN/Perl>).
 
 
@@ -1910,46 +1956,41 @@ the wide possibilities of Andy Wardley's L<Pod::POM> parser.
 
 =back
 
-Thanks 
-to Philippe Bruhat who mentioned a weakness in the API, 
-to Chris Dolan who supplied many useful suggestions and patches 
-(esp. integration with AnnoCPAN), 
-to Rmi Pauchet who pointed out a regression bug with Firefox CSS, 
+Thanks
+to Philippe Bruhat who mentioned a weakness in the API,
+to Chris Dolan who supplied many useful suggestions and patches
+(esp. integration with AnnoCPAN),
+to Rmi Pauchet who pointed out a regression bug with Firefox CSS,
 to Alexandre Jousset who fixed a bug in the TOC display,
 to Cdric Bouvier who pointed out a IO bug in serving binary files,
 to Elliot Shank who contributed the "page_title" option,
-and to Olivier 'dolmen' Mengu who suggested to export "server" into C<main::>.
+to Olivier 'dolmen' Mengu who suggested to export "server" into C<main::>,
+to Ben Bullock who added the 403 message for absent modules,
+and to Paul Cochrane for several improvements in the doc and in the
+repository structure.
 
 
 =head1 RELEASE NOTES
 
-Indexed information since version 1.04 is not compatible 
+Indexed information since version 1.04 is not compatible
 with previous versions.
 
-So if you upgraded from a previous version and want to use 
-the index, you need to rebuild it entirely, by running the 
+So if you upgraded from a previous version and want to use
+the index, you need to rebuild it entirely, by running the
 command :
 
   perl -MPod::POM::Web::Indexer -e "index(-from_scratch => 1)"
 
 
-=head1 BUGS
-
-Please report any bugs or feature requests to
-C<bug-pod-pom-web at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Pod-POM-Web>.
-I will be notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
-
 
 =head1 AUTHOR
 
-Laurent Dami, C<< <laurent.d...@justice.ge.ch> >>
+Laurent Dami, C<< <dami AT cpan DOT org> >>
 
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2007-2017 Laurent Dami, all rights reserved.
+Copyright 2007-2021 Laurent Dami, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -1959,12 +2000,12 @@ under the same terms as Perl itself.
   - real tests !
   - factorization (esp. initial <head> in html pages)
   - use Getopts to choose colouring package, toggle CPAN, etc.
-  - declare Pod::POM bugs 
-      - perlre : line 1693 improper parsing of L<C<< (?>pattern) >>> 
+  - declare Pod::POM bugs
+      - perlre : line 1693 improper parsing of L<C<< (?>pattern) >>>
    - bug: doc files taken as pragmas (lwptut, lwpcook, pip, pler)
    - exploit doc index X<...>
    - do something with perllocal (installation history)
    - restrict to given set of paths/ modules
-       - ned to change toc (no perlfunc, no scripts/pragmas, etc)
+       - need to change toc (no perlfunc, no scripts/pragmas, etc)
        - treenav with letter entries or not ?
   - port to Plack
